@@ -1,5 +1,3 @@
-#!/usr/bin/env python
-
 """Usage:
   client.py run [-s | --single-threaded | --socket SOCKET] [--path PATH]
                   [--non-interactive] [--log LOGFILE] [--scp]
@@ -26,9 +24,7 @@ Options:
   --version              : Show version information.
 """
 
-
 from deployer import __version__
-from deployer.contrib.default_config import example_settings
 from deployer.run.socket_client import list_sessions
 from deployer.run.socket_client import start as start_client
 from deployer.run.socket_server import start as start_server
@@ -37,13 +33,15 @@ from deployer.run.telnet_server import start as start_telnet_server
 
 import docopt
 import getpass
+import os
 import sys
 
-def start(root_service, name=sys.argv[0], extra_loggers=None):
+
+def start(root_node, name=sys.argv[0], extra_loggers=None):
     """
     Client startup point.
     """
-    a = docopt.docopt(__doc__.replace('client.py', name), version=__version__)
+    a = docopt.docopt(__doc__.replace('client.py', os.path.basename(name)), version=__version__)
 
     # "client.py scp" is a shorthand for "client.py run -s --scp"
     if a['scp']:
@@ -72,12 +70,12 @@ def start(root_service, name=sys.argv[0], extra_loggers=None):
     # Telnet server
     elif a['telnet-server']:
         port = int(a['PORT']) if a['PORT'] is not None else 23
-        start_telnet_server(root_service, logfile=a['--log'], port=port,
+        start_telnet_server(root_node, logfile=a['--log'], port=port,
                 extra_loggers=extra_loggers)
 
     # Socket server
     elif a['listen']:
-        socket_name = start_server(root_service, daemonized=False,
+        socket_name = start_server(root_node, daemonized=False,
                     shutdown_on_last_disconnect=False,
                     interactive=interactive, logfile=a['--log'], socket=a['--socket'],
                     extra_loggers=extra_loggers)
@@ -89,7 +87,7 @@ def start(root_service, name=sys.argv[0], extra_loggers=None):
 
     # Single threaded client
     elif a['run'] and a['--single-threaded']:
-        start_standalone(root_service, interactive=interactive, cd_path=path,
+        start_standalone(root_node, interactive=interactive, cd_path=path,
                 action_name=action, parameters=parameters, logfile=a['--log'],
                 extra_loggers=extra_loggers, open_scp_shell=scp)
 
@@ -98,13 +96,9 @@ def start(root_service, name=sys.argv[0], extra_loggers=None):
         # If no socket has been given. Start a daemonized server in the
         # background, and use that socket instead.
         if not socket_name:
-            socket_name = start_server(root_service, daemonized=True,
+            socket_name = start_server(root_node, daemonized=True,
                     shutdown_on_last_disconnect=True, interactive=interactive,
                     logfile=a['--log'], extra_loggers=extra_loggers)
 
         start_client(socket_name, path, action_name=action, parameters=parameters,
                 open_scp_shell=scp)
-
-
-if __name__ == '__main__':
-    start(root_service=example_settings)
