@@ -23,10 +23,11 @@ __all__ = (
     'IsolationIdentifierType',
     'Node',
     'NodeBase',
-    'SimpleNode',
-    'SimpleNodeBase',
+    'ParallelActionResult',
     'ParallelNode',
     'ParallelNodeBase',
+    'SimpleNode',
+    'SimpleNodeBase',
     'iter_isolations',
     'required_property',
 )
@@ -205,7 +206,9 @@ class Env(object):
         Create a default environment for this node to run.
 
         It will be attached to stdin/stdout and commands will be logged to
-        stdout. The is the most obvious default.
+        stdout. The is the most obvious default to create an ``Env`` instance.
+
+        :param node: :class:`~deployer.node.base.Node` instance
         """
         from deployer.pseudo_terminal import Pty
         from deployer.loggers import LoggerInterface
@@ -647,7 +650,10 @@ class Node(object):
 
     @_internal
     def __init__(self, parent=None):
+        #: Reference to the parent :class:`~deployer.node.base.Node`.
+        #: (This is always assigned in the constructor. You should never override it.)
         self.parent = parent
+
         if self._node_type in (NodeTypes.SIMPLE_ARRAY, NodeTypes.SIMPLE_ONE) and not parent:
             raise Exception('Cannot initialize a node of type %s without a parent' % self._node_type)
 
@@ -801,12 +807,20 @@ class ParallelNode(Node):
     Multiple hosts can be given for this role, but all of them will be isolated,
     during execution. This allows parallel executing of functions on each 'cell'.
 
+    If you call a method on a ``ParallelNode``, it will be called one for every
+    host, which can be accessed through the ``host`` property.
+
     :note: This was called `SimpleNode` before.
     """
     __metaclass__ = ParallelNodeBase
     _node_type = NodeTypes.SIMPLE
 
     def host(self):
+        """
+        This is the proxy to the active host.
+
+        :returns: :class:`~deployer.host_container.HostContainer` instance.
+        """
         if self._node_is_isolated:
             host = self.hosts.filter('host')
             if len(host) != 1:
@@ -948,7 +962,7 @@ class EnvAction(object):
         """
         Run the action on one isolation. (On a normal Node, or on a ParallelNode cell.)
         """
-        with isolation._logger.group(self._action._func, *a, **kw):
+        with isolation._logger.group(self._action._func.__name__, *a, **kw):
             while True:
                 try:
                     return self._action._func(isolation, *a, **kw)
@@ -992,6 +1006,8 @@ class EnvAction(object):
                     raise e2
 
     def __call__(self, *a, **kw):
+        # In case of a not_isolated ParallelNode, return a
+        # ParallelActionResult, otherwise, just return the actual result.
         if isinstance(self._env, ParallelNode) and not self._env._node_is_isolated and \
                             not getattr(self._action._func, 'dont_isolate_yet', False):
 
@@ -1000,12 +1016,12 @@ class EnvAction(object):
 
             # No hosts in ParallelNode. Nothing to do.
             if len(isolations) == 0:
-                print 'Nothing to do. No hosts in %r' % self._action
-                return [ ]
+                self._env._logger.log_msg('Nothing to do. No hosts in %r' % self._action)
+                return ParallelActionResult([ ])
 
             # Exactly one host.
             elif len(isolations) == 1:
-                return [ self._run_on_node(isolations[0], *a, **kw) ]
+                return ParallelActionResult([ (isolations[0], self._run_on_node(isolations[0], *a, **kw)) ])
 
             # Multiple hosts, but isolate_one_only flag set.
             elif getattr(self._action._func, 'isolate_one_only', False):
@@ -1035,7 +1051,7 @@ class EnvAction(object):
 
                             # Succeed
                             logger_fork.set_succeeded()
-                            return result
+                            return (isolation, result)
                         except Exception as e:
                             # TODO: handle exception in thread
                             logger_fork.set_failed(e)
@@ -1055,7 +1071,32 @@ class EnvAction(object):
                     raise errors[0]
                 else:
                     # This returns a list of results.
-                    return fork_result.result
+                    return ParallelActionResult(fork_result.result)
         else:
             return self._run_on_node(self._env, *a, **kw)
 
+
+class ParallelActionResult(dict):
+    """
+    When an action of a ParallelNode was called from outside the parallel node
+    itself, a `ParallelActionResult` instance is returned. This contains the
+    result for each isolation.
+
+    (Unconventional, but) Iterating through the `ParallelActionResult` class
+    will yield the values (the results) instead of the keys, because of
+    backwards compatibility and this is typically what people are interested in
+    if they run:
+    ``for result in node.action(...)``.
+
+    The `keys`, `items` and `values` functions work as usual.
+    """
+    def __init__(self, isolations_and_results):
+        # This is a list of (isolation, result) tuples.
+        super(ParallelActionResult, self).__init__(isolations_and_results)
+
+    def __repr__(self):
+        return '{ %s }' % ', '.join('<%s>: %r' % (i.host._host.slug, v) for i, v in self.items())
+
+    def __iter__(self):
+        for v in self.values():
+            yield v
